/* HandleEvent.c
 * 
 * Author: Ben Levitt
 * Last Updated: 7/9/2001
 * Version 1.2
 * 
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the license, or (at your
 * option) any later version.
 *      
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *      
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */



#include <Pilot.h>
#include "Prefs.h"


#define myResID		1000

typedef	Boolean	(*FrmHandleEventInterface)(FormPtr frm, EventPtr eventP);


Boolean MyFrmHandleEvent(FormPtr frm, EventPtr eventP);
static Boolean BetterMenus(FormPtr frm, EventPtr eventP, MenuHackPreferenceType prefs);
static Boolean BetterPopups(FormPtr frm, EventPtr eventP, MenuHackPreferenceType prefs);
static Boolean BetterTablePopups(FormPtr frm, EventPtr eventP, MenuHackPreferenceType prefs);
static ListPtr GetPopupList(FormPtr frm, Word controlID);
static ListPtr GetListByID(FormPtr frm, Word listID);


Boolean MyFrmHandleEvent(FormPtr frm, EventPtr eventP)
{
	FrmHandleEventInterface oldTrap;
	DWord temp;
	MenuHackPreferenceType prefs;
	
	FtrGet(myCreator, myResID, &temp);
	oldTrap = (FrmHandleEventInterface)temp;
	

	// Use the BetterMenus menu-popping Scheme
	if (eventP->eType == frmTitleEnterEvent && frm->menuRscId){
		prefs = GetPrefs();
		if (prefs.betterMenus)
			if (BetterMenus(frm, eventP, prefs))
				return true;
	}

	// Use the BetterPopups menu-popping Scheme
	if (eventP->eType == ctlEnterEvent &&
		((ControlPtr)eventP->data.ctlEnter.pControl)->style == popupTriggerCtl){
		prefs = GetPrefs();
		if (prefs.betterPopups)
			if(BetterPopups(frm, eventP, prefs))
				return true;
	}

	// Use the BetterPopups menu-popping Scheme in popup Table items
	if (eventP->eType == tblEnterEvent){
		prefs = GetPrefs();
		if (prefs.betterPopups)
			if(BetterTablePopups(frm, eventP, prefs))
				return true;
	}
	
	// Thwart OS3.5+'s lo-fi menu-popping scheme
	if (eventP->eType == frmTitleSelectEvent)
		return true;
	
	
	return oldTrap(frm, eventP);
}


static Boolean BetterMenus(FormPtr frm, EventPtr eventP, MenuHackPreferenceType prefs)
{
	MenuBarPtr menu;
	EventType e;
	SWord x, y, newx, newy;
	Boolean down;
	long t;
	Word err;
	Word r, c, choice;
		
		
	EvtGetPen(&x, &y, &down);
	if (x < prefs.width){
		menu = MenuGetActiveMenu();
		if (! menu) {
			if(DmGetResource('MBAR', frm->menuRscId)){
               	menu = MenuInit(frm->menuRscId);
                   MenuSetActiveMenu(menu);
               }
           }
           
		if (! menu)
			return false;
		
		MenuDrawMenu(menu);
		SndPlaySystemSound(sndClick);
		EvtGetPen(&x, &y, &down);
		t = TimGetTicks();
		
		while (TimGetTicks() < t + prefs.delay){
			EvtGetEvent(&e, 5);
			if (e.eType == penUpEvent)
				return true;
			if (e.eType == penMoveEvent){
				EvtGetPen(&newx, &newy, &down);
				if (newx >= x+prefs.drag || newx+prefs.drag <= x ||
					newy >= y+prefs.drag || newy+prefs.drag <= y)
						break;
			}
		}
		
		e.eType = penDownEvent;
		e.penDown = true;
		e.screenX = 5;
		e.screenY = 5;
		MenuHandleEvent(0, &e, &err);
		
		return true;
	}

	return false;
}


static Boolean BetterPopups(FormPtr frm, EventPtr eventP, MenuHackPreferenceType prefs)
{
	EventType e;
	SWord x, y, newx, newy;
	Boolean down;
	long t;
	Word err;
	ListPtr list;
	
	EvtGetPen(&x, &y, &down);		
	t = TimGetTicks();
	
	list = GetPopupList(frm, eventP->data.ctlEnter.controlID);
	// Careful... list may be NULL
	
	while (TimGetTicks() < t + prefs.delay){
		EvtGetEvent(&e, 5);
		if (e.eType == penUpEvent)
			return false;
			
		if (e.eType == penMoveEvent){
			EvtGetPen(&newx, &newy, &down);
			if (newx >= x+prefs.drag || newx+prefs.drag <= x ||
				newy >= y+prefs.drag || newy+prefs.drag <= y)
					break;
		}
	}
	
	SndPlaySystemSound(sndClick);
	e.eType = ctlSelectEvent;
	e.data.ctlSelect.controlID = ((ControlPtr)eventP->data.ctlEnter.pControl)->id;
	e.data.ctlSelect.pControl = eventP->data.ctlEnter.pControl;
	e.data.ctlSelect.on = false;
	EvtAddEventToQueue(&e);
	
	e.eType = penDownEvent;
	e.penDown = true;
	
	if(list){
		e.screenX = list->bounds.topLeft.x+1;
		e.screenY = list->bounds.topLeft.y+1;
	}
	else{
		e.screenX = x;
		e.screenY = y;
	}
	EvtAddEventToQueue(&e);
	
	return true;
}

static Boolean BetterTablePopups(FormPtr frm, EventPtr eventP, MenuHackPreferenceType prefs)
{
	EventType e;
	SWord x, y, newx, newy;
	Boolean down;
	long t;
	Word err;
	ListPtr list;
	Word r, c, choice;
	TablePtr table;
	TableItemType *item;
	
	r = eventP->data.tblEnter.row;
	c = eventP->data.tblEnter.column;
	table = (TablePtr)eventP->data.tblEnter.pTable;
	item = &(table->items[r * table->numColumns + c]);
	
	if (item->itemType == popupTriggerTableItem){
		EvtGetPen(&x, &y, &down);		
		t = TimGetTicks();
		
		list = (ListPtr)(item->ptr);
		// Careful... list may be NULL
		
		while (TimGetTicks() < t + prefs.delay){
			EvtGetEvent(&e, 5);
			if (e.eType == penUpEvent)
				return false;

			if (e.eType == penMoveEvent){
				EvtGetPen(&newx, &newy, &down);
				if (newx >= x+prefs.drag || newx+prefs.drag <= x ||
					newy >= y+prefs.drag || newy+prefs.drag <= y)
						break;
			}
		}
		
		e.eType = penDownEvent;
		e.penDown = true;
		
		if(list){
			e.screenX = list->bounds.topLeft.x+1;
			e.screenY = list->bounds.topLeft.y+1;
		}
		EvtAddEventToQueue(&e);

		SndPlaySystemSound(sndClick);
		choice = LstPopupList(list);
		if (choice == -1){
			e.eType = tblExitEvent;
			e.data.tblSelect.tableID = table->id;
			e.data.tblSelect.pTable = table;
			e.data.tblSelect.row = r;
			e.data.tblSelect.column = c;
			EvtAddEventToQueue(&e);

			return true;
		}
		
		TblSetItemInt(table, r, c, choice);
		TblMarkRowInvalid(table, r);
		TblRedrawTable(table);
		
		e.eType = tblSelectEvent;
		e.data.tblSelect.tableID = table->id;
		e.data.tblSelect.pTable = table;
		e.data.tblSelect.row = r;
		e.data.tblSelect.column = c;
		EvtAddEventToQueue(&e);
		
		return true;
	}
	
	return false;
}


static ListPtr GetPopupList(FormPtr frm, Word triggerID)
{
	Word i,n;
	
	n = FrmGetNumberOfObjects(frm);
	
	for (i=0; i<n; i++)
		if (FrmGetObjectType(frm, i) == frmPopupObj)
			if (((FormPopupType *)FrmGetObjectPtr(frm, i))->controlID == triggerID)
				return GetListByID(frm, ((FormPopupType *)FrmGetObjectPtr(frm, i))->listID);

	//else  (shouldn't occur)
	return NULL;
}

static ListPtr GetListByID(FormPtr frm, Word listID)
{
	Word i,n;
	
	n = FrmGetNumberOfObjects(frm);
	
	for (i=0; i<n; i++)
		if (FrmGetObjectType(frm, i) == frmListObj)
			if (((ListPtr)FrmGetObjectPtr(frm, i))->id == listID)
				return FrmGetObjectPtr(frm, i);

	return NULL;
}
